@@ -10,8 +10,8 @@ from GoogleScraper.utils import chunk_it
 from GoogleScraper.commandline import get_command_line
 from GoogleScraper.database import ScraperSearch, SERP, Link, get_session, fixtures
 from GoogleScraper.proxies import parse_proxy_file, get_proxies_from_mysql_db, add_proxies_to_db
-from GoogleScraper.http import HttpScrape
-from GoogleScraper.selenium import SelScrape, get_selenium_scraper_by_search_engine_name
+from GoogleScraper.http_mode import HttpScrape
+from GoogleScraper.selenium_mode import SelScrape, get_selenium_scraper_by_search_engine_name
 from GoogleScraper.caching import fix_broken_cache_names, _caching_is_one_to_one, parse_all_cached_files, clean_cachefiles
 from GoogleScraper.config import InvalidConfigurationException, parse_cmd_args, Config, update_config_with_file
 from GoogleScraper.log import out
@@ -203,7 +203,7 @@ def main(return_results=False, parse_cmd_line=True):
 
     if Config['GLOBAL'].getboolean('shell', False):
         namespace = {}
-        Session = get_session(scoped=False, create=False)
+        Session = get_session(scoped=False)
         namespace['session'] = Session()
         namespace['ScraperSearch'] = ScraperSearch
         namespace['SERP'] = SERP
