@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
 import sys
+import os
 import re
-import hashlib
 import lxml.html
 from lxml.html.clean import Cleaner
 import logging
@@ -41,11 +41,21 @@ class Parser():
     Attributes:
         search_results: The results after parsing.
     """
+
+    # if subclasses specify an value for this attribute and the attribute
+    # targets an element in the serp page, then there weren't any results
+    # for the original query.
+    no_results_selectors = []
+
+    # the selector that gets the number of results (guessed) as shown by the search engine.
+    num_results_search_selectors = []
+
+    # some search engine show on which page we currently are. If supportd, this selector will get this value.
+    page_number_selectors = []
     
     # The supported search types. For instance, Google supports Video Search, Image Search, News search
     search_types = []
 
-
     # Each subclass of Parser may declare an arbitrary amount of attributes that
     # follow a naming convention like this:
     # *_search_selectors
@@ -72,18 +82,24 @@ class Parser():
         self.html = html
         self.dom = None
         self.search_results = {}
-        self.search_results['num_results'] = ''
+        self.num_results = ''
+        self.effective_query = ''
+        self.page_number = -1
+
+        # short alias because we use it so extensively
+        self.css_to_xpath = HTMLTranslator().css_to_xpath
         
         if self.html:
             self.parse()
         
-    def parse(self, html):
+    def parse(self, html=None):
         """Public function to start parsing the search engine results.
         
         Args: 
             html: The raw html data to extract the SERP entries from.
         """
-        self.html = html
+        if html:
+            self.html = html
         
         # lets do the actual parsing
         self._parse()
@@ -107,7 +123,7 @@ class Parser():
     def _parse(self, cleaner=None):
         """Internal parse the dom according to the provided css selectors.
         
-        Raises: InvalidSearchTypeExcpetion if no css selectors for the searchtype could be found.
+        Raises: InvalidSearchTypeException if no css selectors for the searchtype could be found.
         """
         self._parse_lxml(cleaner)
         
@@ -115,22 +131,20 @@ class Parser():
         attr_name = self.searchtype + '_search_selectors'
         selector_dict = getattr(self, attr_name, None)
 
-        # short alias because we use it so extensively
-        css_to_xpath = HTMLTranslator().css_to_xpath
-
         # get the appropriate css selectors for the num_results for the keyword
         num_results_selector = getattr(self, 'num_results_search_selectors', None)
-        self.search_results['num_results'] = ''
 
-        if isinstance(num_results_selector, list) and num_results_selector:
-            for selector in num_results_selector:
-                try:
-                    self.search_results['num_results'] = self.dom.xpath(css_to_xpath(selector))[0].text_content()
-                except IndexError as e:
-                    logger.warning('Cannot parse num_results from serp page with selector {}'.format(selector))
-                else: # leave when first selector grabbed something
-                    break
+        self.num_results = self.first_match(num_results_selector, self.dom)
+        if not self.num_results:
+            logger.warning('{}: Cannot parse num_results from serp page with selectors {}'.format(self.__class__.__name__, num_results_selector))
 
+        # get the current page we are at. Sometimes we search engines don't show this.
+        self.page_number = self.first_match(self.page_number_selectors, self.dom)
+
+        # let's see if the search query was shitty (no results for that query)
+        self.effective_query = self.first_match(self.no_results_selectors, self.dom)
+
+        # get the stuff that is of interest in SERP pages.
         if not selector_dict and not isinstance(selector_dict, dict):
             raise InvalidSearchTypeException('There is no such attribute: {}. No selectors found'.format(attr_name))
 
@@ -140,9 +154,15 @@ class Parser():
 
             for selector_specific, selectors in selector_class.items():
 
+                if 'result_container' in selectors and selectors['result_container']:
+                    css = '{container} {result_container}'.format(**selectors)
+                else:
+                    css = selectors['container']
+
                 results = self.dom.xpath(
-                    css_to_xpath('{container} {result_container}'.format(**selectors))
+                    self.css_to_xpath(css)
                 )
+
                 to_extract = set(selectors.keys()) - {'container', 'result_container'}
                 selectors_to_use = {key: selectors[key] for key in to_extract if key in selectors.keys()}
 
@@ -159,26 +179,8 @@ class Parser():
                     # key are for example 'link', 'snippet', 'visible-url', ...
                     # selector is the selector to grab these items
                     for key, selector in selectors_to_use.items():
-                        value = None
-                        if selector.endswith('::text'):
-                            try:
-                                value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].text_content()
-                            except IndexError as e:
-                                pass
-                        else:
-                            attr = re.search(r'::attr\((?P<attr>.*)\)$', selector).group('attr')
-                            if attr:
-                                try:
-                                    value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].get(attr)
-                                except IndexError as e:
-                                    pass
-                            else:
-                                try:
-                                    value = result.xpath(css_to_xpath(selector))[0].text_content()
-                                except IndexError as e:
-                                    pass
+                        serp_result[key] = self.advanced_css(selector, result)
 
-                        serp_result[key] = value
                     serp_result['rank'] = index+1
 
                     # only add items that have not None links.
@@ -188,22 +190,66 @@ class Parser():
                             not [e for e in self.search_results[result_type] if e['link'] == serp_result['link']]:
                         self.search_results[result_type].append(serp_result)
 
-    def result_id(self, args):
-        """Gets an unique id for the args.
+
+    def advanced_css(self, selector, element):
+        """Evaluate the :text and ::attr(attr-name) additionally.
 
         Args:
-            args: A number of arguments that constitute the uniqueness of the result.
+            selector: A css selector.
+            element: The element on which to apply the selector.
 
         Returns:
-            An hash of the concatenated arguments.
+            The targeted element.
+
         """
-        hasher = hashlib.md5()
-        for arg in args:
-            if arg:
-                hasher.update(arg.encode())
+        value = None
+
+        if selector.endswith('::text'):
+            try:
+                value = element.xpath(self.css_to_xpath(selector.split('::')[0]))[0].text_content()
+            except IndexError as e:
+                pass
+        else:
+            match = re.search(r'::attr\((?P<attr>.*)\)$', selector)
+
+            if match:
+                attr = match.group('attr')
+                try:
+                    value = element.xpath(self.css_to_xpath(selector.split('::')[0]))[0].get(attr)
+                except IndexError as e:
+                    pass
+            else:
+                try:
+                    value = element.xpath(self.css_to_xpath(selector))[0].text_content()
+                except IndexError as e:
+                    pass
+
+        return value
+
+
+    def first_match(self, selectors, element):
+        """Get the first match.
+
+        Args:
+            selectors: The selectors to test for a match.
+            element: The element on which to apply the selectors.
+
+        Returns:
+            The very first match or False if all selectors didn't match anything.
+        """
+        assert isinstance(selectors, list), 'selectors must be a list type you motherfucker'
+
+        for selector in selectors:
+            if selector:
+                try:
+                    match = self.advanced_css(selector, element=element)
+                    if match:
+                        return match
+                except IndexError as e:
+                    pass
+
+        return False
 
-        return hasher.hexdigest()
-                    
     def after_parsing(self):
         """Subclass specific behaviour after parsing happened.
         
@@ -227,6 +273,16 @@ class Parser():
         self.dom = cleaner.clean_html(self.dom)
         assert len(self.dom), 'The html needs to be parsed to get the cleaned html'
         return lxml.html.tostring(self.dom)
+
+
+    def iter_serp_items(self):
+        """Yields the key and index of any item in the serp results if it has a link value"""
+
+        for key, value in self.search_results.items():
+            if isinstance(value, list):
+                for i, item in enumerate(value):
+                    if isinstance(item, dict) and item['link']:
+                        yield (key, i)
                 
 """
 Here follow the different classes that provide CSS selectors 
@@ -258,8 +314,12 @@ class GoogleParser(Parser):
     """Parses SERP pages of the Google search engine."""
     
     search_types = ['normal', 'image']
-    
+
+    no_results_selectors = ['#topstuff .med > b::text']
+
     num_results_search_selectors = ['#resultStats']
+
+    page_number_selectors = ['#navcnt td.cur::text']
     
     normal_search_selectors = {
         'results': {
@@ -278,7 +338,14 @@ class GoogleParser(Parser):
                 'snippet': 'div.s span.st::text',
                 'title': 'h3.r > a:first-child::text',
                 'visible_link': 'cite::text'
-            }
+            },
+            'de_ip_news_items': {
+                'container': 'li.card-section',
+                'link': 'a._Dk::attr(href)',
+                'snippet': 'span._dwd::text',
+                'title': 'a._Dk::text',
+                'visible_link': 'cite::text'
+            },
         },
         'ads_main': {
             'us_ip': {
@@ -338,16 +405,13 @@ class GoogleParser(Parser):
             'image': r'imgres\?imgurl=(?P<url>.*?)&'
         }
 
-        for key, value in self.search_results.items():
-            if isinstance(value, list):
-                for i, item in enumerate(value):
-                    if isinstance(item, dict) and item['link']:
-                        result = re.search(
-                            clean_regexes[self.searchtype],
-                            item['link']
-                        )
-                        if result:
-                            self.search_results[key][i]['link'] = unquote(result.group('url'))
+        for key, i in self.iter_serp_items():
+            result = re.search(
+                clean_regexes[self.searchtype],
+                self.search_results[key][i]['link']
+            )
+            if result:
+                self.search_results[key][i]['link'] = unquote(result.group('url'))
 
 
 class YandexParser(Parser):
@@ -355,7 +419,7 @@ class YandexParser(Parser):
 
     search_types = ['normal', 'image']
     
-    num_results_search_selectors = []
+    num_results_search_selectors = ['.serp-item__wrap strong']
     
     normal_search_selectors = {
         'results': {
@@ -402,18 +466,15 @@ class YandexParser(Parser):
         super().after_parsing()
 
         if self.searchtype == 'image':
-            for key, value in self.search_results.items():
-                if isinstance(value, list):
-                    for i, item in enumerate(value):
-                        if isinstance(item, dict) and item['link']:
-                            for regex in (
-                                r'\{"href"\s*:\s*"(?P<url>.*?)"\}',
-                                r'img_url=(?P<url>.*?)&'
-                            ):
-                                result = re.search(regex, item['link'])
-                                if result:
-                                    self.search_results[key][i]['link'] = result.group('url')
-                                    break
+            for key, i in self.iter_serp_items():
+                for regex in (
+                    r'\{"href"\s*:\s*"(?P<url>.*?)"\}',
+                    r'img_url=(?P<url>.*?)&'
+                ):
+                    result = re.search(regex, self.search_results[key][i]['link'])
+                    if result:
+                        self.search_results[key][i]['link'] = result.group('url')
+                        break
     
     
 class BingParser(Parser):
@@ -429,7 +490,7 @@ class BingParser(Parser):
                 'container': '#b_results',
                 'result_container': '.b_algo',
                 'link': 'h2 > a::attr(href)',
-                'snippet': '.b_caption > .b_attribution > p::text',
+                'snippet': '.b_caption > p::text',
                 'title': 'h2::text',
                 'visible_link': 'cite::text'
             },
@@ -440,7 +501,14 @@ class BingParser(Parser):
                 'snippet': '.b_caption > p::text',
                 'title': 'h2::text',
                 'visible_link': 'cite::text'
-            }
+            },
+            'de_ip_news_items': {
+                'container': 'ul.b_vList li',
+                'link': ' h5 a::attr(href)',
+                'snippet': 'p::text',
+                'title': ' h5 a::text',
+                'visible_link': 'cite::text'
+            },
         },
         'ads_main': {
             'us_ip': {
@@ -488,17 +556,14 @@ class BingParser(Parser):
         super().after_parsing()
 
         if self.searchtype == 'image':
-            for key, value in self.search_results.items():
-                if isinstance(value, list):
-                    for i, item in enumerate(value):
-                        if isinstance(item, dict) and item['link']:
-                            for regex in (
-                                r'imgurl:"(?P<url>.*?)"',
-                            ):
-                                result = re.search(regex, item['link'])
-                                if result:
-                                    self.search_results[key][i]['link'] = result.group('url')
-                                    break
+            for key, i in self.iter_serp_items():
+                for regex in (
+                    r'imgurl:"(?P<url>.*?)"',
+                ):
+                    result = re.search(regex, self.search_results[key][i]['link'])
+                    if result:
+                        self.search_results[key][i]['link'] = result.group('url')
+                        break
 
 
 class YahooParser(Parser):
@@ -550,19 +615,20 @@ class YahooParser(Parser):
         """
         super().after_parsing()
 
+        for key, i in self.iter_serp_items():
+            if self.search_results[key][i]['visible_link'] is None:
+                del self.search_results[key][i]
+
         if self.searchtype == 'image':
-            for key, value in self.search_results.items():
-                if isinstance(value, list):
-                    for i, item in enumerate(value):
-                        if isinstance(item, dict) and item['link']:
-                            for regex in (
-                                r'&imgurl=(?P<url>.*?)&',
-                            ):
-                                result = re.search(regex, item['link'])
-                                if result:
-                                    # TODO: Fix this manual protocol adding by parsing "rurl"
-                                    self.search_results[key][i]['link'] = 'http://' + unquote(result.group('url'))
-                                    break
+            for key, i in self.iter_serp_items():
+                for regex in (
+                    r'&imgurl=(?P<url>.*?)&',
+                ):
+                    result = re.search(regex, self.search_results[key][i]['link'])
+                    if result:
+                        # TODO: Fix this manual protocol adding by parsing "rurl"
+                        self.search_results[key][i]['link'] = 'http://' + unquote(result.group('url'))
+                        break
 
 class BaiduParser(Parser):
     """Parses SERP pages of the Baidu search engine."""
@@ -580,6 +646,14 @@ class BaiduParser(Parser):
                 'snippet': '.c-abstract::text',
                 'title': 'h3 > a.t::text',
                 'visible_link': 'span.c-showurl::text'
+            },
+            'nojs': {
+                'container': '#content_left',
+                'result_container': '.result',
+                'link': 'h3 > a::attr(href)',
+                'snippet': '.c-abstract::text',
+                'title': 'h3 > a::text',
+                'visible_link': 'span.g::text'
             }
         },
     }
@@ -609,17 +683,14 @@ class BaiduParser(Parser):
         super().after_parsing()
 
         if self.searchtype == 'image':
-            for key, value in self.search_results.items():
-                if isinstance(value, list):
-                    for i, item in enumerate(value):
-                        if isinstance(item, dict) and item['link']:
-                            for regex in (
-                                r'&objurl=(?P<url>.*?)&',
-                            ):
-                                result = re.search(regex, item['link'])
-                                if result:
-                                    self.search_results[key][i]['link'] = unquote(result.group('url'))
-                                    break
+            for key, i in self.iter_serp_items():
+                for regex in (
+                    r'&objurl=(?P<url>.*?)&',
+                ):
+                    result = re.search(regex, self.search_results[key][i]['link'])
+                    if result:
+                        self.search_results[key][i]['link'] = unquote(result.group('url'))
+                        break
 
 
 class DuckduckgoParser(Parser):
@@ -781,7 +852,8 @@ def parse_serp(html=None, search_engine=None,
                 requested_at=requested_at,
                 requested_by=requested_by,
                 query=current_keyword,
-                num_results_for_keyword=parser.search_results['num_results'],
+                effective_query=parser.effective_query,
+                num_results_for_keyword=parser.num_results,
             )
 
         for key, value in parser.search_results.items():
@@ -809,7 +881,7 @@ def parse_serp(html=None, search_engine=None,
 
         serp.num_results = num_results
 
-        return (serp, parser)
+        return serp
 
 if __name__ == '__main__':
     """Originally part of https://github.com/NikolaiT/GoogleScraper.
@@ -825,14 +897,18 @@ if __name__ == '__main__':
     But for some engines it nevertheless works (for example: yandex, google, ...).
     """
     import requests
-    assert len(sys.argv) == 2, 'Usage: {} url'.format(sys.argv[0])
+    assert len(sys.argv) >= 2, 'Usage: {} url/file'.format(sys.argv[0])
     url = sys.argv[1]
-    raw_html = requests.get(url).text
-    parser = get_parser_by_url(url)
+    if os.path.exists(url):
+        raw_html = open(url, 'r').read()
+        parser = get_parser_by_search_engine(sys.argv[2])
+    else:
+        raw_html = requests.get(url).text
+        parser = get_parser_by_url(url)
+
     parser = parser(raw_html)
     parser.parse()
     print(parser)
     
     with open('/tmp/testhtml.html', 'w') as of:
         of.write(raw_html)
-    
